import { Reflector } from '@nestjs/core'
import {
  Injectable,
  CanActivate,
  ExecutionContext,
  applyDecorators,
  UseGuards,
  SetMetadata,
  ForbiddenException,
} from '@nestjs/common'
import { EmailScene, EmailSendDto } from './dto/email.dto'
import { EmailService } from './email.service'
import { JwtService } from '@nestjs/jwt'
import { type Request } from 'express'

const EMAIL_SCENE_KEY = 'EMAIL_SCENE'

@Injectable()
class EmailVeriyGuard implements CanActivate {
  constructor(
    private emailService: EmailService,
    private reflector: Reflector,
    private jwtService: JwtService,
  ) {}

  canActivate(ctx: ExecutionContext) {
    const scene = this.reflector.get<EmailScene>(
      EMAIL_SCENE_KEY,
      ctx.getHandler(),
    )

    const request = ctx.switchToHttp().getRequest<Request>()
    const { token, email, verifyCode } = request.body

    const payload = this.jwtService.verify<EmailSendDto>(token, {
      secret: this.emailService.getEmailJwtSecret(verifyCode, scene),
    })
    if (payload.email !== email || payload.scene !== scene) {
      throw new ForbiddenException('请输入正确的邮箱验证码')
    }
    return true
  }
}

export function VerifyEmail(scene: EmailScene) {
  return applyDecorators(
    SetMetadata(EMAIL_SCENE_KEY, scene),
    UseGuards(EmailVeriyGuard),
  )
}
